@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -90,7 +90,7 @@ sub UserForAuthString {
 
 =head1 NAME
 
-RT-Authen-Token - token-based authentication
+RT::Authen::Token - token-based authentication
 
 =head1 DESCRIPTION
 
