@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -723,7 +723,7 @@ sub DeleteValue {
     return ($ok, $self->loc("Custom field value deleted"));
 }
 
-=head2 ValidateValue Value
+=head2 ValidateValue VALUE
 
 Make sure that the supplied value is valid
 
@@ -736,7 +736,7 @@ sub ValidateValue {
     return $ret;
 }
 
-=head2 ValidateQueue Queue
+=head2 ValidateName NAME
 
 Make sure that the name specified is valid
 
@@ -751,7 +751,7 @@ sub ValidateName {
     return $self->SUPER::ValidateName($value);
 }
 
-=head2 ValidateQueue Queue
+=head2 ValidateQueue QUEUE
 
 Make sure that the queue specified is a valid queue name
 
@@ -1203,7 +1203,7 @@ sub _Value {
 =head2 SetDisabled
 
 Takes a boolean.
-1 will cause this custom field to no longer be avaialble for objects.
+1 will cause this custom field to no longer be available for objects.
 0 will re-enable this field.
 
 =cut
@@ -2137,7 +2137,7 @@ sub LinkValueTo {
 With one argument, returns the _URLTemplate named C<NAME>, but only if
 the current user has the right to see this custom field.
 
-With two arguments, attemptes to set the relevant template value.
+With two arguments, attempts to set the relevant template value.
 
 =cut
 
@@ -2381,6 +2381,26 @@ sub CleanupDefaultValues {
     }
 }
 
+=head2 IsNumeric
+
+Returns true if the custom field is supposed to be numeric, default is 0.
+
+Right now you need to override this method to mark the chosen ones numeric.
+
+=cut
+
+sub IsNumeric { 0 }
+
+=head2 NumericPrecision
+
+Returns the precision if the custom field is numeric, default is C<undef>.
+
+Right now you need to override this method to customize it.
+
+=cut
+
+sub NumericPrecision { undef }
+
 =head2 id
 
 Returns the current value of id. 
