@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -479,22 +479,18 @@ sub ObsoleteSubscription {
     }
 }
 
-sub EmailDashboard {
+sub DashboardSubject {
     my $self = shift;
     my %args = (
         CurrentUser  => undef,
-        Email        => undef,
         Dashboard    => undef,
         Subscription => undef,
-        Content      => undef,
         @_,
     );
 
     my $subscription = $args{Subscription};
     my $dashboard    = $args{Dashboard};
     my $currentuser  = $args{CurrentUser};
-    my $email        = $args{Email};
-
     my $frequency    = $subscription->SubValue('Frequency');
 
     my %frequency_lookup = (
@@ -512,6 +508,31 @@ sub EmailDashboard {
         $currentuser->loc($frequency_display),
         $dashboard->Name;
 
+    return $subject;
+}
+
+sub EmailDashboard {
+    my $self = shift;
+    my %args = (
+        CurrentUser  => undef,
+        Email        => undef,
+        Dashboard    => undef,
+        Subscription => undef,
+        Content      => undef,
+        @_,
+    );
+
+    my $subscription = $args{Subscription};
+    my $dashboard    = $args{Dashboard};
+    my $currentuser  = $args{CurrentUser};
+    my $email        = $args{Email};
+
+    my $subject = $self->DashboardSubject(
+        CurrentUser  => $currentuser,
+        Dashboard    => $dashboard,
+        Subscription => $subscription,
+    );
+
     my $entity = $self->BuildEmail(
         %args,
         To      => $email,
