@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -109,7 +109,7 @@ determines the user that the mail was sent from, and performs the given
 actions.
 
 Returns a list of C<(status, message, ticket)>.  The C<status> is -75
-for a temporary failure (to be retried later bt the MTA), 0 for a
+for a temporary failure (to be retried later by the MTA), 0 for a
 permanent failure which did not result in a ticket, and 1 for a ticket
 that was found and acted on.
 
@@ -368,7 +368,7 @@ sub GetCurrentUser {
 
 =head3 CheckACL Action => C<action>, CurrentUser => C<user>, Ticket => C<ticket>, Queue => C<queue>
 
-Checks that the currentuser can perform a particular action.  While RT's
+Checks that the CurrentUser can perform a particular action.  While RT's
 standard permission controls apply, this allows a better error message,
 or more limited restrictions on the email gateway.
 
@@ -566,7 +566,7 @@ sub CheckForLoops {
 =head3 CheckForAutoGenerated HEAD
 
 Takes a HEAD object of L<MIME::Head> class and returns true if message is
-autogenerated. Checks C<Precedence>, RFC3834 C<Auto-Submitted>, and
+auto-generated. Checks C<Precedence>, RFC3834 C<Auto-Submitted>, and
 C<X-FC-Machinegenerated> fields of the head in tests.
 
 =cut
@@ -786,7 +786,7 @@ sub _OutgoingMailFrom {
 
 Sends an email (passed as a L<MIME::Entity> object C<ENTITY>) using
 RT's outgoing mail configuration. If C<BOUNCE> is passed, and is a
-true value, the message will be marked as an autogenerated error, if
+true value, the message will be marked as an auto-generated error, if
 possible. Sets Date field of the head to now if it's not set.
 
 If the C<X-RT-Squelch> header is set to any true value, the mail will
@@ -1293,7 +1293,7 @@ sub DeleteRecipientsFromHead {
 Takes a hash with a String and a Charset. Returns the string encoded
 according to RFC2047, using B (base64 based) encoding.
 
-String must be a perl string, octets are returned.
+String must be a Perl string, octets are returned.
 
 If Charset is not provided then $EmailOutputEncoding config option
 is used, or "latin-1" if that is not set.
