@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -198,7 +198,7 @@ dry run (import is false) then function shouldn't change anything.
 
 =item mapping
 
-Hash reference with the currently processed mapping, eg. C<$LDAPMapping>.
+Hash reference with the currently processed mapping, e.g. C<$LDAPMapping>.
 
 =item rt_field and ldap_field
 
