@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -404,7 +404,7 @@ Takes status and returns list of statuses it can be changed to.
 Is status is empty or undefined then returns list of statuses for
 a new ticket.
 
-If argument is ommitted then returns a hash with all possible
+If argument is omitted then returns a hash with all possible
 transitions in the following format:
 
     status_x => [ next_status, next_status, ... ],
@@ -625,9 +625,12 @@ sub CanonicalCase {
     return($self->{data}{canonical_case}{lc $status} || lc $status);
 }
 
+our @WARNINGS;
 sub FillCache {
     my $self = shift;
 
+    @WARNINGS = ();
+
     my $map = RT->Config->Get('Lifecycles') or return;
 
     {
@@ -643,6 +646,7 @@ sub FillCache {
             for my $name ( @lifecycles ) {
                 unless ( $map->{$name} ) {
                     warn "Lifecycle $name is missing in %Lifecycles config";
+                    push @WARNINGS, loc( "Lifecycle [_1] is missing in %Lifecycles config", $name );
                 }
             }
         }
@@ -667,6 +671,7 @@ sub FillCache {
         my ( $ret, @warnings ) = $self->ValidateLifecycle(Lifecycle => $lifecycle, Name => $name);
         unless ( $ret ) {
             warn $_ for @warnings;
+            push @WARNINGS, @warnings;
         }
 
         my @statuses;
@@ -739,6 +744,7 @@ sub FillCache {
     my ( $ret, @warnings ) = $self->ValidateLifecycleMaps();
     unless ( $ret ) {
         warn $_ for @warnings;
+        push @WARNINGS, @warnings;
     }
 
     # Lower-case the transition maps
@@ -799,6 +805,22 @@ sub _SaveLifecycles {
     my $lifecycles = shift;
     my $CurrentUser = shift;
 
+    for my $key ( keys %$lifecycles ) {
+        next if $key eq '__maps__';
+        my ( $ret, @warnings )
+            = $class->ValidateLifecycle( Lifecycle => $lifecycles->{$key}, Name => $key, Cleanup => 1 );
+        unless ($ret) {
+            RT->Logger->debug($_) for @warnings;
+        }
+    }
+
+    if ( $lifecycles->{__maps__} ) {
+        my ( $ret, @warnings ) = $class->ValidateLifecycleMaps( Lifecycles => $lifecycles, Cleanup => 1 );
+        unless ($ret) {
+            RT->Logger->debug($_) for @warnings;
+        }
+    }
+
     my $setting = RT::Configuration->new($CurrentUser);
     $setting->LoadByCols(Name => 'Lifecycles', Disabled => 0);
     if ($setting->Id) {
@@ -823,13 +845,13 @@ sub _CreateLifecycle {
     my %args  = @_;
     my $CurrentUser = $args{CurrentUser};
 
-    my $lifecycles = RT->Config->Get('Lifecycles');
+    my %lifecycles = RT->Config->Get('Lifecycles');
     my $lifecycle;
 
     if ($args{Clone}) {
-        $lifecycle = Storable::dclone($lifecycles->{ $args{Clone} });
+        $lifecycle = Storable::dclone($lifecycles{ $args{Clone} });
         $class->_CloneLifecycleMaps(
-            $lifecycles->{__maps__},
+            $lifecycles{__maps__},
             $args{Name},
             $args{Clone},
         );
@@ -838,9 +860,9 @@ sub _CreateLifecycle {
         $lifecycle = { type => $args{Type} };
     }
 
-    $lifecycles->{$args{Name}} = $lifecycle;
+    $lifecycles{$args{Name}} = $lifecycle;
 
-    my ($ok, $msg) = $class->_SaveLifecycles($lifecycles, $CurrentUser);
+    my ($ok, $msg) = $class->_SaveLifecycles(\%lifecycles, $CurrentUser);
     return ($ok, $msg) if !$ok;
 
     return (1, $CurrentUser->loc("Lifecycle [_1] created", $args{Name}));
@@ -889,6 +911,85 @@ sub CreateLifecycle {
     return $class->_CreateLifecycle(%args);
 }
 
+=head2 DeleteLifecycle( CurrentUser => undef, Name => undef )
+
+Delete a lifecycle.
+
+Returns (STATUS, MESSAGE). STATUS is true if succeeded, otherwise false.
+
+=cut
+
+sub DeleteLifecycle {
+    my $class = shift;
+    my %args  = (
+        CurrentUser => undef,
+        Name        => undef,
+        @_,
+    );
+
+    my $CurrentUser = $args{CurrentUser};
+    my $Name        = $args{Name};
+
+    return ( 0, $CurrentUser->loc("Lifecycle Name required") ) unless length $Name;
+
+    my $lifecycles = RT->Config->Get('Lifecycles');
+    my ( $ok, $msg );
+    if ( $lifecycles->{$Name} ) {
+        my $dep_class   = ( $lifecycles->{$Name}{'type'} // '' ) eq 'asset' ? 'RT::Catalogs' : 'RT::Queues';
+        my $dep_objects = $dep_class->new( $args{CurrentUser} );
+        $dep_objects->Limit( FIELD => 'Lifecycle', VALUE => $Name );
+        my @dep_names = map { $_->Name } @{ $dep_objects->ItemsArrayRef };
+
+        if (@dep_names) {
+            return (
+                0,
+                $CurrentUser->loc(
+                    "Could not delete lifecycle [_1]: it is used by [_2] [_3]",
+                    $Name,
+                    (
+                        $dep_class eq 'RT::Queues'
+                        ? ( @dep_names > 1 ? 'queues'   : 'queue' )
+                        : ( @dep_names > 1 ? 'catalogs' : 'catalog' )
+                    ),
+                    join( ', ', @dep_names ),
+                )
+            );
+        }
+        else {
+            delete $lifecycles->{$Name};
+            my $maps = $lifecycles->{__maps__};
+            for my $key ( keys %$maps ) {
+                if ( $key =~ m/^$Name -> / || $key =~ m/ -> $Name$/ ) {
+                    delete $maps->{$key};
+                }
+            }
+            ( $ok, $msg ) = $class->_SaveLifecycles( $lifecycles, $CurrentUser );
+            return ( $ok, $msg ) if !$ok && $msg ne $CurrentUser->loc( '[_1] update: Nothing changed', 'Lifecycles' );
+        }
+    }
+
+    my $meta = RT->Config->Meta('Lifecycles');
+    my @file_sources = grep { !$_->{Database} && exists $_->{Value}{$Name} } @{ $meta->{Sources} };
+    if ( @file_sources ) {
+        my $message = $ok
+            ? "Lifecycle '[_1]' deleted from database. To delete this lifecycle, you must also remove it from the following config file: [_2]" # loc
+            : "To delete '[_1]', you must remove it from the following config file: [_2]"; # loc
+        return (
+            0,
+            $CurrentUser->loc(
+                $message,  $args{Name},
+                join ', ', map {"$_->{File} line $_->{Line}"} @file_sources
+            )
+        );
+    }
+    elsif ( defined $ok && !$ok ) {
+        return ( $ok, $msg );
+    }
+
+    return ( 1, $CurrentUser->loc( 'Lifecycle [_1] deleted', $args{Name} ) );
+}
+
+
 =head2 UpdateLifecycle( CurrentUser => undef, LifecycleObj => undef, NewConfig => undef, Maps => undef )
 
 Update passed lifecycle to the new configuration.
@@ -923,12 +1024,15 @@ sub UpdateLifecycle {
     return (1, $CurrentUser->loc("Lifecycle [_1] updated", $name));
 }
 
-=head2 UpdateMaps( CurrentUser => undef, Maps => undef )
+=head2 UpdateMaps( CurrentUser => undef, Maps => undef, Name => undef )
 
 Update lifecycle maps.
 
 Returns (STATUS, MESSAGE). STATUS is true if succeeded, otherwise false.
 
+Note that the Maps in argument will be merged into existing maps. To delete
+all existing items for a lifecycle before merging, pass its Name.
+
 =cut
 
 sub UpdateMaps {
@@ -936,14 +1040,19 @@ sub UpdateMaps {
     my %args = (
         CurrentUser  => undef,
         Maps         => undef,
+        Name         => undef,
         @_,
     );
 
     my $CurrentUser = $args{CurrentUser};
     my $lifecycles = RT->Config->Get('Lifecycles');
 
+    my $all_maps = $lifecycles->{__maps__} || {};
+    # When Name is absent, we want to keep all items
+    my @keep_keys = grep { !$args{Name} || !/^\Q$args{Name}\E -> | -> \Q$args{Name}\E$/ } keys %$all_maps;
+
     %{ $lifecycles->{__maps__} } = (
-        %{ $lifecycles->{__maps__} || {} },
+        map( { $_ => $all_maps->{$_} } @keep_keys ),
         %{ $args{Maps} },
     );
 
@@ -953,10 +1062,13 @@ sub UpdateMaps {
     return (1, $CurrentUser->loc("Lifecycle mappings updated"));
 }
 
-=head2 ValidateLifecycle( CurrentUser => undef, Lifecycle => undef, Name => undef )
+=head2 ValidateLifecycle( CurrentUser => undef, Lifecycle => undef, Name => undef, Cleanup => undef )
 
 Validate passed Lifecycle data structure.
 
+If Cleanup is true, clean up passed Lifecycle data structure, e.g. removing
+nonexistent statuses.
+
 Returns (STATUS, MESSAGE). STATUS is true if succeeded, otherwise false.
 
 =cut
@@ -967,6 +1079,7 @@ sub ValidateLifecycle {
         CurrentUser => undef,
         Lifecycle   => undef,
         Name        => undef,
+        Cleanup     => undef,
         @_,
     );
     my $current_user = $args{CurrentUser} || RT->SystemUser;
@@ -995,43 +1108,79 @@ sub ValidateLifecycle {
     for my $state ( keys %{ $lifecycle->{defaults} || {} } ) {
         my $status = $lifecycle->{defaults}{$state};
         if ( $status ) {
-            push @warnings, $current_user->loc( "Nonexistant default [_1] status [_2] in [_3] lifecycle", $state, lc $status, $name )
-                unless $lifecycle->{canonical_case}{ lc $status };
+            unless ( $lifecycle->{canonical_case}{ lc $status } ) {
+                push @warnings, $current_user->loc( "Nonexistent default [_1] status [_2] in [_3] lifecycle", $state, lc $status, $name );
+                if ( $args{Cleanup} ) {
+                    delete $lifecycle->{defaults}{$state};
+                }
+            }
         }
         else {
             push @warnings, $current_user->loc( "Empty default [_1] status in [_2] Lifecycle", $state, $name );
         }
     }
     for my $from ( keys %{ $lifecycle->{transitions} || {} } ) {
-        push @warnings, $current_user->loc( "Nonexistant status [_1] in transitions in [_2] lifecycle", lc $from, $name )
-            unless $from eq '' || $lifecycle->{canonical_case}{ lc $from };
+        unless ( $from eq '' || $lifecycle->{canonical_case}{ lc $from } ) {
+            push @warnings,
+                $current_user->loc( "Nonexistent status [_1] in transitions in [_2] lifecycle", lc $from, $name );
+            if ( $args{Cleanup} ) {
+                delete $lifecycle->{transitions}{$from};
+                next;
+            }
+        }
 
         for my $status ( @{ ( $lifecycle->{transitions}{$from} ) || [] } ) {
-            push @warnings, $current_user->loc( "Nonexistant status [_1] in transitions in [_2] lifecycle", lc $status, $name )
+            push @warnings, $current_user->loc( "Nonexistent status [_1] in transitions in [_2] lifecycle", lc $status, $name )
                 unless $lifecycle->{canonical_case}{ lc $status };
         }
+
+        # Remove nonexistent statuses
+        if ( $args{Cleanup} ) {
+            $lifecycle->{transitions}{$from}
+                = [ grep { $lifecycle->{canonical_case}{ lc $_ } } @{ ( $lifecycle->{transitions}{$from} ) || [] } ];
+        }
     }
 
     for my $schema ( keys %{ $lifecycle->{rights} || {} } ) {
         my ( $from, $to ) = split /\s*->\s*/, $schema, 2;
         unless ( $from and $to ) {
             push @warnings, $current_user->loc( "Invalid right transition [_1] in [_2] lifecycle", $schema, $name );
+            if ( $args{Cleanup} ) {
+                delete $lifecycle->{rights}{$schema};
+            }
             next;
         }
-        push @warnings, $current_user->loc( "Nonexistant status [_1] in right transition in [_2] lifecycle", lc $from, $name )
-            unless $from eq '*'
-            or $lifecycle->{canonical_case}{ lc $from };
-        push @warnings, $current_user->loc( "Nonexistant status [_1] in right transition in [_2] lifecycle", lc $to, $name )
-            unless $to eq '*' || $lifecycle->{canonical_case}{ lc $to };
 
-        push @warnings,
-            $current_user->loc( "Invalid right name ([_1]) in [_2] lifecycle; right names must be ASCII", $lifecycle->{rights}{$schema}, $name )
-            if $lifecycle->{rights}{$schema} =~ /\P{ASCII}/;
+        my $invalid;
+        unless ( $from eq '*' or $lifecycle->{canonical_case}{ lc $from } ) {
+            push @warnings,
+                $current_user->loc( "Nonexistent status [_1] in right transition in [_2] lifecycle", lc $from, $name );
+            $invalid = 1;
+        }
 
-        push @warnings,
-            $current_user
-            ->loc( "Invalid right name ([_1]) in [_2] lifecycle; right names must be <= 25 characters", $lifecycle->{rights}{$schema}, $name )
-            if length( $lifecycle->{rights}{$schema} ) > 25;
+        unless ( $to eq '*' or $lifecycle->{canonical_case}{ lc $to } ) {
+            push @warnings,
+                $current_user->loc( "Nonexistent status [_1] in right transition in [_2] lifecycle", lc $to, $name );
+            $invalid = 1;
+        }
+
+        if ( $lifecycle->{rights}{$schema} =~ /\P{ASCII}/ ) {
+            push @warnings,
+                $current_user->loc( "Invalid right name ([_1]) in [_2] lifecycle; right names must be ASCII",
+                    $lifecycle->{rights}{$schema}, $name );
+            $invalid = 1;
+        }
+
+        if ( length( $lifecycle->{rights}{$schema} ) > 25 ) {
+            push @warnings,
+                $current_user->loc( "Invalid right name ([_1]) in [_2] lifecycle; right names must be <= 25 characters",
+                $lifecycle->{rights}{$schema}, $name );
+            $invalid = 1;
+        }
+
+        if ( $args{Cleanup} && $invalid ) {
+            delete $lifecycle->{rights}{$schema};
+        }
     }
 
     my @actions;
@@ -1044,27 +1193,44 @@ sub ValidateLifecycle {
         @actions = @{ $lifecycle->{'actions'} };
     }
 
+    my @valid_actions;
     while ( my ( $transition, $info ) = splice @actions, 0, 2 ) {
         my ( $from, $to ) = split /\s*->\s*/, $transition, 2;
         unless ( $from and $to ) {
             push @warnings, $current_user->loc( "Invalid action status change [_1], in [_2] lifecycle", $transition, $name );
             next;
         }
-        push @warnings, $current_user->loc( "Nonexistant status [_1] in actions in [_2] lifecycle", lc $from, $name )
-            unless $from eq '*'
-            or $lifecycle->{canonical_case}{ lc $from };
-        push @warnings, $current_user->loc( "Nonexistant status [_1] in actions in [_2] lifecycle", lc $to, $name )
-            unless $to eq '*'
-            or $lifecycle->{canonical_case}{ lc $to };
+
+        my $invalid;
+        unless ( $from eq '*' or $lifecycle->{canonical_case}{ lc $from } ) {
+            push @warnings, $current_user->loc( "Nonexistent status [_1] in actions in [_2] lifecycle", lc $from, $name );
+            $invalid = 1;
+        }
+
+        unless ( $to eq '*' or $lifecycle->{canonical_case}{ lc $to } ) {
+            push @warnings, $current_user->loc( "Nonexistent status [_1] in actions in [_2] lifecycle", lc $to, $name );
+            $invalid = 1;
+        }
+
+        if ( $args{Cleanup} && !$invalid ) {
+            push @valid_actions, $transition, $info;
+        }
+    }
+
+    if ( $args{Cleanup} ) {
+        $lifecycle->{'actions'} = \@valid_actions;
     }
 
     return @warnings ? ( 0, uniq @warnings ) : 1;
 }
 
-=head2 ValidateLifecycleMaps( CurrentUser => undef )
+=head2 ValidateLifecycleMaps( CurrentUser => undef, Lifecycles => undef, Maps => undef, Cleanup => undef )
 
 Validate lifecycle Maps.
 
+If Cleanup is true, clean up maps structure, e.g. removing nonexistent
+statuses.
+
 Returns (STATUS, MESSAGES). STATUS is true if succeeded, otherwise false.
 
 =cut
@@ -1073,32 +1239,57 @@ sub ValidateLifecycleMaps {
     my $self = shift;
     my %args = (
         CurrentUser => undef,
+        Lifecycles  => undef,
+        Maps        => undef,
+        Cleanup     => undef,
         @_,
     );
     my $current_user = $args{CurrentUser} || RT->SystemUser;
 
     my @warnings;
-    for my $mapname ( keys %{ $LIFECYCLES_CACHE{'__maps__'} || {} } ) {
+    my $lifecycles = $args{Lifecycles} || \%LIFECYCLES_CACHE;
+    my $maps       = $args{Maps} || $lifecycles->{__maps__} || $LIFECYCLES_CACHE{__maps__} || {};
+
+    for my $mapname ( keys %$maps ) {
         my ( $from, $to ) = split /\s*->\s*/, $mapname, 2;
         unless ( $from and $to ) {
             push @warnings, $current_user->loc( "Invalid lifecycle mapping [_1]", $mapname );
+            if ( $args{Cleanup} ) {
+                delete $maps->{$mapname};
+            }
             next;
         }
-        push @warnings, $current_user->loc( "Nonexistant lifecycle [_1] in [_2] lifecycle map", $from, $mapname )
-            unless $LIFECYCLES_CACHE{$from};
-        push @warnings, $current_user->loc( "Nonexistant lifecycle [_1] in [_2] lifecycle map", $to, $mapname )
-            unless $LIFECYCLES_CACHE{$to};
+        push @warnings, $current_user->loc( "Nonexistent lifecycle [_1] in [_2] lifecycle map", $from, $mapname )
+            unless $lifecycles->{$from};
+        push @warnings, $current_user->loc( "Nonexistent lifecycle [_1] in [_2] lifecycle map", $to, $mapname )
+            unless $lifecycles->{$to};
 
         # Ignore mappings referring to disabled lifecycles
         next if $LIFECYCLES_CACHE{$from} && $LIFECYCLES_CACHE{$from}{disabled};
         next if $LIFECYCLES_CACHE{$to} && $LIFECYCLES_CACHE{$to}{disabled};
 
-        my $map = $LIFECYCLES_CACHE{'__maps__'}{$mapname};
+        if ( $args{Cleanup} && ( !$lifecycles->{$from} || !$lifecycles->{$to} ) ) {
+            delete $maps->{$mapname};
+            next;
+        }
+
+        my $map = $maps->{$mapname};
+        my $new_map = {};
         for my $status ( keys %{$map} ) {
-            push @warnings, $current_user->loc( "Nonexistant status [_1] in [_2] in [_3] lifecycle map", lc $status, $from, $mapname )
-                if $LIFECYCLES_CACHE{$from} && !$LIFECYCLES_CACHE{$from}{canonical_case}{ lc $status };
-            push @warnings, $current_user->loc( "Nonexistant status [_1] in [_2] in [_3] lifecycle map", lc $map->{$status}, $to, $mapname )
-                if $LIFECYCLES_CACHE{$to} && !$LIFECYCLES_CACHE{$to}{canonical_case}{ lc $map->{$status} };
+            push @warnings, $current_user->loc( "Nonexistent status [_1] in [_2] in [_3] lifecycle map", lc $status, $from, $mapname )
+                if $lifecycles->{$from} && !$lifecycles->{$from}{canonical_case}{ lc $status };
+            push @warnings, $current_user->loc( "Nonexistent status [_1] in [_2] in [_3] lifecycle map", lc $map->{$status}, $to, $mapname )
+                if $lifecycles->{$to} && !$lifecycles->{$to}{canonical_case}{ lc $map->{$status} };
+            $new_map->{$status} = $map->{$status}
+                if $args{Cleanup}
+                && $lifecycles->{$from}
+                && $lifecycles->{$from}{canonical_case}{ lc $status }
+                && $lifecycles->{$to}
+                && $lifecycles->{$to}{canonical_case}{ lc $map->{$status} };
+        }
+
+        if ( $args{Cleanup} ) {
+            $maps->{$mapname} = $new_map;
         }
     }
 
