@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -135,7 +135,7 @@ sub run {
     # Plack::Handler::FCGI has its own catch for this, but doesn't
     # notice that listen is an empty list, and we can also provide a
     # better error message.
-    if ($self->{server} eq "FCGI" and not -S STDIN and not @{$args{listen} || []}) {
+    if (!$ENV{SERVER_STARTER_PORT} and $self->{server} eq "FCGI" and not -S STDIN and not @{$args{listen} || []}) {
         print STDERR "STDIN is not a socket, and no --listen, --socket, or --port provided\n";
         exit 1;
     }
