@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -350,7 +350,7 @@ response/comment, mandatory.
 The reason why you should encode the content of any file to C<MIME Base64>
 is that a JSON string value should be a sequence of zero or more Unicode
 characters. C<MIME Base64> is a binary-to-text encoding scheme widely used
-(for eg. by web browser) to send binary data when text data is required.
+(for e.g. by web browser) to send binary data when text data is required.
 Most popular language have C<MIME Base64> libraries that you can use to
 encode the content of your attached files (see L<MIME::Base64> for C<Perl>).
 Note that even text files should be C<MIME Base64> encoded to be passed in
@@ -472,7 +472,7 @@ services that your team uses.
 
 Currently provided endpoints under C</REST/2.0/> are described below.
 Wherever possible please consider using C<_hyperlinks> hypermedia
-controls available in response bodies rather than hardcoding URLs.
+controls available in response bodies rather than hard-coding URLs.
 
 For simplicity, the examples below omit the extra options to
 curl for SSL like --cacert.
@@ -737,7 +737,7 @@ a single ticket id or an array.
     GET /asset/:id/history
         retrieve list of transactions for asset
 
-=head3 Assets Examples
+=head3 Asset Examples
 
 Below are some examples using the endpoints above.
 
@@ -748,8 +748,8 @@ Below are some examples using the endpoints above.
 
     # Search Assets
     curl -X POST -H "Content-Type: application/json" -u 'root:password'
-    -d '[{ "field" : "id", "operator" : ">=", "value" : 0 }]'
-    'https://myrt.com/REST/2.0/assets'
+        -d '[{ "field" : "id", "operator" : ">=", "value" : 0 }]'
+        'https://myrt.com/REST/2.0/assets'
 
     # Search Assets Based On Custom Field Values using L</JSON searches>
     curl -X POST -H "Content-Type: application/json" -u 'root:password'
@@ -760,20 +760,6 @@ Below are some examples using the endpoints above.
     curl -X POST -u 'root:password' -d "query=Catalog='General assets' AND 'CF.{Asset Type}' LIKE 'Computer'"
         'https://myrt.com/REST/2.0/assets'
 
-=head3 Assets Examples
-
-Below are some examples using the endpoints above.
-
-    # Create an Asset
-    curl -X POST -H "Content-Type: application/json" -u 'root:password'
-        -d '{"Name" : "Asset From Rest", "Catalog" : "General assets", "Content" : "Some content"}'
-        'https://myrt.com/REST/2.0/asset'
-
-    # Search Assets
-    curl -X POST -H "Content-Type: application/json" -u 'root:password'
-    -d '[{ "field" : "id", "operator" : ">=", "value" : 0 }]'
-    'https://myrt.com/REST/2.0/assets'
-
 =head3 Catalogs
 
     GET /catalogs/all
@@ -1052,7 +1038,7 @@ C<order> query parameter will be treated as C<order=ASC>, for ascending
 order. The order of the C<order> query parameters should be the same as
 the C<orderby> query parameters. Therefore, if you specify two fields to
 sort the results (with two C<orderby> parameters) and you want to sort
-the second field by descending order, you should also explicitely
+the second field by descending order, you should also explicitly
 specify C<order=ASC> for the first field:
 C<orderby=Created&order=ASC&orderby=id&order=DESC>. C<orderby> and
 C<order> query parameters are supported in both JSON and TicketSQL
@@ -1120,7 +1106,7 @@ some custom fields attached to, you can specify the value(s) for these
 customfields in the C<CustomFields> property of the JSON object parameter.
 The C<CustomFields> property should be a JSON object, with each property
 being the custom field identifier or name. If the custom field can have only
-one value, you just have to speciy the value as JSON string for this custom
+one value, you just have to specify the value as JSON string for this custom
 field. If the customfield can have several value, you have to specify a JSON
 array of each value you want for this custom field.
 
@@ -1176,7 +1162,7 @@ The content, I<encoded in C<MIME Base64>> of the file to attach, mandatory.
 The reason why you should encode the content of the image or binary file to
 C<MIME Base64> is that a JSON string value should be a sequence of zero or
 more Unicode characters. C<MIME Base64> is a binary-to-text encoding scheme
-widely used (for eg. by web browser) to send binary data when text data is
+widely used (for e.g. by web browser) to send binary data when text data is
 required. Most popular language have C<MIME Base64> libraries that you can
 use to encode the content of your attached files (see L<MIME::Base64> for
 C<Perl>). Note that even text files should be C<MIME Base64> encoded to be
@@ -1205,7 +1191,7 @@ passed in the C<FileContent> property.
 Encoding the content of image or binary files in C<MIME Base64> has the
 drawback of adding some processing overhead and to increase the sent data
 size by around 33%. RT's REST2 API provides another way to upload image or
-binary files as custom field alues by sending, instead of a JSON request, a
+binary files as custom field values by sending, instead of a JSON request, a
 C<multipart/form-data> request. This kind of request is similar to what the
 browser sends when you upload a file in RT's ticket creation or update
 forms. As its name suggests, a C<multipart/form-data> request message
@@ -1222,7 +1208,7 @@ ticket. Files can then be attached by specifying a field named as specified
 in the C<CustomFields> property for each of them, with the content of the
 file as value and the appropriate MIME type.
 
-Here is an exemple of a curl invocation, wrapped to multiple lines for
+Here is an example of a curl invocation, wrapped to multiple lines for
 readability, to create a ticket with a multipart/request to upload some
 image or binary files as custom fields values.
 
@@ -1289,6 +1275,11 @@ or previous page, then the URL for that page is returned in the next_page
 and prev_page variables respectively. It is up to you to store the required
 JSON to pass with the following page request.
 
+If current user doesn't have enough rights to see all returned results, then
+returned C<pages> and C<total> will be C<null>, in which case you can still
+loop through results by means of C<next_page> that contains the link to the next
+page of results, until it becomes absent.
+
 =head2 Disabled items
 
 By default, only enabled objects are returned. To include disabled objects
@@ -1297,7 +1288,7 @@ you can specify C<find_disabled_rows=1> as a query parameter.
 =head2 Fields
 
 When fetching search results you can include additional fields by adding
-a query parameter C<fields> which is a comma seperated list of fields
+a query parameter C<fields> which is a comma separated list of fields
 to include. You must use the camel case version of the name as included
 in the results for the actual item.
 
@@ -1305,7 +1296,7 @@ You can use additional fields parameters to expand child blocks, for
 example (line wrapping inserted for readability):
 
     XX_RT_URL_XX/REST/2.0/tickets
-      ?fields=Owner,Status,Created,Subject,Queue,CustomFields,Requestor,Cc,AdminCc,RT::CustomRole-1
+      ?fields=Owner,Status,Created,Subject,Queue,CustomFields,Requestor,Cc,AdminCc,CustomRoles
       &fields[Queue]=Name,Description
 
 Says that in the result set for tickets, the extra fields for Owner, Status,
@@ -1360,13 +1351,13 @@ ticket is displayed in this example):
             }
          ],
          "AdminCc" : [],
-         "RT::CustomRole-1" : [
-            {
+         "CustomRoles" : {
+            "Manager": {
                "_url" : "XX_RT_URL_XX/REST/2.0/user/foo@example.com",
                "type" : "user",
                "id" : "foo@example.com"
             }
-         ]
+         }
       }
       { … },
       …
