@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -64,9 +64,17 @@ my @tokens = qw[VALUE AGGREGATOR OPERATOR OPEN_PAREN CLOSE_PAREN KEYWORD];
 use Regexp::Common qw /delimited/;
 my $re_aggreg      = qr[(?i:AND|OR)];
 my $re_delim       = qr[$RE{delimited}{-delim=>qq{\'\"}}];
-my $re_value       = qr[[+-]?\d+|(?i:NULL)|$re_delim];
-my $re_keyword     = qr[[{}\w\.]+|$re_delim];
-my $re_op          = qr[=|!=|>=|<=|>|<|(?i:IS NOT)|(?i:IS)|(?i:NOT LIKE)|(?i:LIKE)|(?i:NOT STARTSWITH)|(?i:STARTSWITH)|(?i:NOT ENDSWITH)|(?i:ENDSWITH)]; # long to short
+
+# We need to support bare(not quoted) strings like CF.{Beta Date} to use the
+# content of related custom field as the value to compare, e.g.
+#
+#       Due < CF.{Beta Date}
+#
+# Support it in keyword part is mainly for consistency.
+
+my $re_value       = qr[(?i:CF)\.\{.+?\}(?:\.(?i:Content|LargeContent))?|[\w\.]+|[+-]?\d+|(?i:NULL)|$re_delim];
+my $re_keyword     = qr[(?i:CF)\.\{.+?\}(?:\.(?i:Content|LargeContent))?|[{}\w\.]+|$re_delim];
+my $re_op          = qr[(?i:SHALLOW )?(?:=|!=|>=|<=|>|<|(?i:IS NOT)|(?i:IS)|(?i:NOT LIKE)|(?i:LIKE)|(?i:NOT STARTSWITH)|(?i:STARTSWITH)|(?i:NOT ENDSWITH)|(?i:ENDSWITH))]; # long to short
 my $re_open_paren  = qr[\(];
 my $re_close_paren = qr[\)];
 
@@ -169,7 +177,26 @@ sub Parse {
                 s!\\(.)!$1!g;
             }
 
-            $cb->{'Condition'}->( $key, $op, $value );
+            my $quote_value;
+            if ( $match =~ /$re_delim/o ) {
+                $quote_value = 1;
+
+                # It's really rare to search strings like "CF.foo", to DWIM,
+                # automatically convert to columns
+                if ( $value =~ /^CF\.(?:\{(.*)\}|(.*?))(?:\.(Content|LargeContent))?$/i ) {
+                    RT->Logger->debug("Unquote value($match) to search custom field instead");
+                    $quote_value = 0;
+                }
+            }
+            elsif ( $match =~ /^[a-z]/i ) {
+                # Value is a column
+                $quote_value = 0;
+            }
+            else {
+                # Not setting value here to fallback to default behavior
+            }
+
+            $cb->{'Condition'}->( $key, $op, $value, $quote_value );
 
             ($key,$op,$value) = ("","","");
             $want = AGGREG;
@@ -212,6 +239,85 @@ sub _BitmaskToString {
     return join ' or ', @res;
 }
 
+sub _Optimize {
+    my $tree = shift;
+
+    # Convert simple OR'd clauses to IN for better performance, e.g.
+    #     (Status = 'new' OR Status = 'open' OR Status = 'stalled')
+    # to
+    #     Status IN ('new', 'open', 'stalled')
+
+    $tree->traverse(
+        sub {
+            my $node   = shift;
+            my $parent = $node->getParent;
+            return if $parent eq 'root';    # Skip root's root
+
+            # For simple searches like "Status = 'new' OR Status = 'open'",
+            # the OR node is also the root node, go up one level.
+            $node = $parent if $node->isLeaf && $parent->isRoot;
+
+            return if $node->isLeaf;
+
+            if ( ( $node->getNodeValue // '' ) =~ /^or$/i && $node->getChildCount > 1 ) {
+                my @children = $node->getAllChildren;
+                my %info;
+                for my $child (@children) {
+
+                    # Only handle innermost ORs
+                    return unless $child->isLeaf;
+                    my $entry = $child->getNodeValue;
+                    return unless $entry->{Op} =~ /^!?=$/;
+
+                    # Handle String/Int/Id/Enum/Queue/Lifecycle only for
+                    # now. Others have more complicated logic inside, which
+                    # can't be easily converted.
+
+                    # TICKETQUEUEFIELD only supports Lifecycle right now, which is fine
+                    return
+                        unless ( $entry->{Meta}[0] // '' )
+                        =~ /^(?:STRING|INT|ID|ENUM|QUEUE|LIFECYCLE|TICKETFIELD|TICKETQUEUEFIELD)$/;
+
+                    # TICKETFIELD contains more than what we want, need to filter a bit more deeply.
+                    if ( $entry->{Meta}[0] eq 'TICKETFIELD' ) {
+                        my ($field) = $entry->{Key} =~ /Ticket(\w+)/;
+                        require RT::Tickets;
+                        return
+                            unless $RT::Tickets::FIELD_METADATA{$field}[0]
+                            =~ /^(?:STRING|INT|ID|ENUM|QUEUE|LIFECYCLE)$/;
+                    }
+
+                    for my $field (qw/Key SubKey Op Value/) {
+                        $info{$field}{ $entry->{$field} // '' } ||= 1;
+
+                        if ( $field eq 'Value' ) {
+
+                            # In case it's meta value like __Bookmarked__
+                            return if $entry->{Meta}[0] eq 'ID' && $entry->{$field} !~ /^\d+$/;
+                        }
+                        elsif ( keys %{ $info{$field} } > 1 ) {
+                            return;    # Skip if Key/SubKey/Op are different
+                        }
+                    }
+                }
+
+                my $first_child = shift @children;
+                my $entry       = $first_child->getNodeValue;
+                $entry->{Op} = $info{Op}{'='} ? 'IN' : 'NOT IN';
+                $entry->{Value} = [ sort keys %{ $info{Value} } ];
+                if ( $node->isRoot ) {
+                    $parent->removeChild($_) for @children;
+                }
+                else {
+                    $parent->removeChild($node);
+                    $parent->addChild($first_child);
+                }
+            }
+        }
+    );
+    return $tree;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
