@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -252,6 +252,11 @@ our @SUPPORTED_OBJECTS = qw(
     Ticket
     Transaction
     User
+    Class
+    Article
+    Topic
+    Catalog
+    Asset
 );
 
 =head3 GENERIC
@@ -307,9 +312,9 @@ sub _Init
 
 =head4 CastObjectsToRecords( Objects => undef )
 
-Cast objects to the C<RT::Record> objects or its ancesstors.
+Cast objects to the C<RT::Record> objects or its ancestors.
 Objects can be passed as SCALAR (format C<< <class>-<id> >>),
-ARRAY, C<RT::Record> ancesstors or C<RT::SearchBuilder> ancesstor.
+ARRAY, C<RT::Record> ancestors or C<RT::SearchBuilder> ancestor.
 
 Most methods that takes C<Objects> argument use this method to
 cast argument value to list of records.
@@ -366,7 +371,7 @@ sub CastObjectsToRecords
         RT::Shredder::Exception->throw( "Unsupported class $class" )
               unless $class =~ /^\w+(::\w+)*$/;
         $class = 'RT::'. $class unless $class =~ /^RTx?::/i;
-        $class->require or die "Failed to load $class: $@";
+        RT::StaticUtil::RequireModule($class) or die "Failed to load $class: $@";
         my $obj = $class->new( RT->SystemUser );
         die "Couldn't construct new '$class' object" unless $obj;
         $obj->Load( $id );
@@ -422,8 +427,8 @@ sub PutObjects
 
 Puts record object into cache and returns its cache entry.
 
-B<NOTE> that this method support B<only C<RT::Record> object or its ancesstor
-objects>, if you want put mutliple objects or objects represented by different
+B<NOTE> that this method support B<only C<RT::Record> object or its ancestor
+objects>, if you want put multiple objects or objects represented by different
 classes then use C<PutObjects> method instead.
 
 =cut
@@ -648,7 +653,7 @@ path by next rules:
 
 * if C<FileName> has C<XXXX> (exactly four uppercase C<X> letters) then it would be changed with digits from 0000 to 9999 range, with first one free value;
 
-* if C<FileName> has C<%T> then it would be replaced with the current date and time in the C<YYYY-MM-DDTHH:MM:SS> format. Note that using C<%t> may still generate not unique names, using C<XXXX> recomended.
+* if C<FileName> has C<%T> then it would be replaced with the current date and time in the C<YYYY-MM-DDTHH:MM:SS> format. Note that using C<%t> may still generate not unique names, using C<XXXX> recommended.
 
 * if C<FromStorage> argument is true (default behaviour) then result path would always be relative to C<StoragePath>;
 
