@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -102,19 +102,19 @@ sub Squish {
 
 =head2 Content
 
-squished content
+Squished content
 
 =head2 Key
 
-md5 of the squished content
+MD5 of the squished content
 
 =head2 ModifiedTime
 
-created time of squished content, i.e. seconds since 00:00:00 UTC, January 1, 1970
+Created time of squished content, i.e. seconds since 00:00:00 UTC, January 1, 1970
 
 =head2 ModifiedTimeString
 
-created time of squished content, with HTTP::Date format
+Created time of squished content, with HTTP::Date format
 
 =cut
 
