@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -118,6 +118,8 @@ custom field.
                                      EmailAddress => 'mycroft@example.com',
                                      UserCF.Relationship => 'Brother' );
 
+If no Name is passed, it is set to the same value as EmailAddress.
+
 =cut
 
 
@@ -204,6 +206,13 @@ sub Create {
         return ( 0, $self->loc('Could not create user') );
     }
 
+    # Add corresponding CustomFieldValue records for custom fields,
+    # the same as in RT::Authen::ExternalAuth::UpdateUserInfo.
+    # RT::Authen::ExternalAuth needs Net::LDAP, which might be unavailable
+    if ( RT::StaticUtil::RequireModule("RT::Authen::ExternalAuth") ) {
+        RT::Authen::ExternalAuth::AddCustomFieldValue(%args);
+    }
+
     # Handle any user CFs
     $self->UpdateObjectCustomFieldValues( %args );
 
@@ -689,7 +698,6 @@ sub LoadOrCreateByEmail {
     return wantarray ? ($self->Id, $self->loc("User loaded")) : $self->Id
         if $self->Id;
 
-    $create{Name}       ||= $create{EmailAddress};
     $create{Privileged} ||= 0;
     $create{Comments}   //= 'Autocreated when added as a watcher';
 
@@ -821,6 +829,20 @@ sub EmailFrequency {
     return '';
 }
 
+=head2 URI
+
+Returns this user's URI
+
+=cut
+
+sub URI {
+    my $self = shift;
+
+    require RT::URI::user;
+    my $uri = RT::URI::user->new($self->CurrentUser);
+    return $uri->URIForObject($self);
+}
+
 =head2 CanonicalizeEmailAddress ADDRESS
 
 CanonicalizeEmailAddress converts email addresses into canonical form.
@@ -834,9 +856,7 @@ is class name not an object.
 sub CanonicalizeEmailAddress {
     my $self = shift;
     my $email = shift;
-    # Example: the following rule would treat all email
-    # coming from a subdomain as coming from second level domain
-    # foo.com
+
     if ( my $match   = RT->Config->Get('CanonicalizeEmailAddressMatch') and
          my $replace = RT->Config->Get('CanonicalizeEmailAddressReplace') )
     {
@@ -872,7 +892,7 @@ sub CanonicalizeUserInfo {
 
 =head2 CanonicalizeUserInfoFromExternalAuth
 
-Convert an ldap entry in to fields that can be used by RT as specified by the
+Convert an LDAP entry in to fields that can be used by RT as specified by the
 C<attr_map> configuration in the C<$ExternalSettings> variable for
 L<RT::Authen::ExternalAuth>.
 
@@ -1022,7 +1042,7 @@ sub SetRandomPassword {
 =head3 ResetPassword
 
 Returns status, [ERROR or new password].  Resets this user's password to
-a randomly generated pronouncable password and emails them, using a
+a randomly generated pronounceable password and emails them, using a
 global template called "PasswordChange".
 
 This function is currently unused in the UI, but available for local scripts.
@@ -1237,15 +1257,18 @@ sub IsPassword {
     }
 
    if ( $self->PrincipalObj->Disabled ) {
+        # Run the bcrypt generator to avoid timing side-channel attacks
+        RT::Util::constant_time_eq($self->_GeneratePassword_bcrypt($value), '0' x 64);
         $RT::Logger->info(
             "Disabled user " . $self->Name . " tried to log in" );
         return (undef);
     }
 
     unless ($self->HasPassword) {
-        return(undef);
-     }
-
+        # Run the bcrypt generator to avoid timing side-channel attacks
+        RT::Util::constant_time_eq($self->_GeneratePassword_bcrypt($value), '0' x 64);
+        return undef;
+    }
     my $stored = $self->__Value('Password');
     if ($stored =~ /^!/) {
         # If it's a new-style (>= RT 4.0) password, it starts with a '!'
@@ -1314,9 +1337,9 @@ sub CurrentUserRequireToSetPassword {
         RequireCurrent => 1,
     );
 
-    if ( RT->Config->Get('WebRemoteUserAuth')
-        && !RT->Config->Get('WebFallbackToRTLogin')
-    ) {
+    if (   ( RT->Config->Get('WebRemoteUserAuth') && !RT->Config->Get('WebFallbackToRTLogin') )
+        || ( RT->Config->Get('ExternalAuth') && !$self->CurrentUser->HasPassword ) )
+    {
         $res{'CanSet'} = 0;
         $res{'Reason'} = $self->loc("External authentication enabled.");
     } elsif ( !$self->CurrentUser->HasPassword ) {
@@ -1336,7 +1359,7 @@ sub CurrentUserRequireToSetPassword {
 
 Returns an authentication string associated with the user. This
 string can be used to generate passwordless URLs to integrate
-RT with services and programms like callendar managers, rss
+RT with services and programs like calendar managers, RSS
 readers and other.
 
 =cut
@@ -1538,20 +1561,24 @@ sub HasGroupRight {
     return ($retval);
 }
 
-=head2 OwnGroups
+=head2 OwnGroups Recursively => 1|0
 
 Returns a group collection object containing the groups of which this
-user is a member.
+user is a member, recursively by default.
 
 =cut
 
 sub OwnGroups {
     my $self = shift;
+    my %args = (
+        Recursively => 1,
+        @_,
+    );
     my $groups = RT::Groups->new($self->CurrentUser);
     $groups->LimitToUserDefinedGroups;
     $groups->WithMember(
         PrincipalId => $self->Id,
-        Recursively => 1
+        Recursively => $args{'Recursively'},
     );
     return $groups;
 }
@@ -1744,7 +1771,7 @@ Returns a list of valid stylesheets take from preferences.
 sub Stylesheet {
     my $self = shift;
 
-    my $style = RT->Config->Get('WebDefaultStylesheet', $self->CurrentUser);
+    my $style = RT->Config->Get('WebDefaultStylesheet', $self);
 
     if (RT::Interface::Web->ComponentPathIsSafe($style)) {
         for my $root (RT::Interface::Web->StaticRoots) {
@@ -2060,7 +2087,7 @@ sub PreferredKey
     return $prefkey->Content if $prefkey;
 
     # we don't have a preferred key for this user, so now we must query GPG
-    my %res = RT::Crypt->GetKeysForEncryption($self->EmailAddress);
+    my %res = RT::Crypt->GetKeysForEncryption(Recipient => $self->EmailAddress, Protocol => 'GnuPG');
     return undef unless defined $res{'info'};
     my @keys = @{ $res{'info'} };
     return undef if @keys == 0;
@@ -2091,7 +2118,26 @@ sub PrivateKey {
     }
 
     my $key = $self->FirstAttribute('PrivateKey') or return undef;
-    return $key->Content;
+    my $content = $key->Content;
+
+    # RT used to identify keys using the key ID, but now identifies them
+    # using the key fingerprint, which is 160 bits long and avoids a
+    # collision attack against keys with short IDs.
+    if ( length $content < 40 ) {
+        # not fingerprint, try to update it
+        my %tmp = RT::Crypt->GetKeysForSigning( Signer => $content, Protocol => 'GnuPG' );
+        if ( !$tmp{exit_code} && $tmp{info} && $tmp{info}[0] ) {
+            my $user = RT::User->new( RT->SystemUser );
+            $user->Load( $self->Id );
+            $user->SetPrivateKey( $tmp{info}[0]{Fingerprint} );
+            return $tmp{info}[0]{Fingerprint};
+        }
+        else {
+            RT->Logger->warning("Couldn't find private key for $content");
+        }
+    }
+
+    return $content;
 }
 
 sub SetPrivateKey {
@@ -2117,6 +2163,8 @@ sub SetPrivateKey {
         my %tmp = RT::Crypt->GetKeysForSigning( Signer => $key, Protocol => 'GnuPG' );
         return (0, $self->loc("No such key or it's not suitable for signing"))
             if $tmp{'exit_code'} || !$tmp{'info'};
+        # In case $key is key id instead of fingerprint
+        $key = $tmp{'info'}[0]{Fingerprint};
     }
 
     my ($status, $msg) = $self->SetAttribute(
@@ -2230,8 +2278,8 @@ sub ToggleBookmark {
 Returns a list of hashrefs { id => <ticket_id>, subject => <ticket_subject> } )
 ordered by recently viewed first.
 
-If a ticket cannot be loaded (eg because it has been shredded) or is duplicated
-(eg because 2 tickets on the list have been merged), it is not returned and the
+If a ticket cannot be loaded (e.g. because it has been shredded) or is duplicated
+(e.g. because 2 tickets on the list have been merged), it is not returned and the
 user's RecentlyViewedTickets list is updated
 
 =cut
@@ -3032,6 +3080,9 @@ sub BeforeWipeout {
 
 sub Serialize {
     my $self = shift;
+    my %args = @_;
+    $args{serializer}{_uid}{user}{ $self->Id } = $self->UID;
+
     return (
         Disabled => $self->PrincipalObj->Disabled,
         Principal => $self->PrincipalObj->UID,
@@ -3072,18 +3123,21 @@ sub PreInflate {
         return;
     }
 
-    # Create a principal first, so we know what ID to use
-    my $principal = RT::Principal->new( RT->SystemUser );
-    my ($id) = $principal->Create(
-        PrincipalType => 'User',
-        Disabled => $disabled,
-    );
+    # serialized data with --all has principals, in which case we don't need to create a new one.
+    my $principal_obj = $importer->LookupObj( $principal_uid );
+    if ( $principal_obj ) {
+        $data->{id} = $principal_obj->Id;
+    }
+    else {
 
-    # Now we have a principal id, set the id for the user record
-    $data->{id} = $id;
+        # Create a principal first, so we know what ID to use
+        my $id = $importer->NextPrincipalId( PrincipalType => 'User', Disabled => $disabled );
 
-    $importer->Resolve( $principal_uid => ref($principal), $id );
-    $data->{id} = $id;
+        # Now we have a principal id, set the id for the user record
+        $data->{id} = $id;
+
+        $importer->Resolve( $principal_uid => 'RT::Principal', $id );
+    }
 
     return $class->SUPER::PreInflate( $importer, $uid, $data );
 }
