@@ -80,6 +80,6 @@ function filter_cascade_select (select, complete_select, vals) {
     }
 
     if ( jQuery(select).closest('div.bootstrap-select').length ) {
-        jQuery(select).selectpicker('refresh');
+        refreshSelectpicker(jQuery(select));
     }
 }
